#include <jni.h>
#include <stdio.h>
#include <winsock2.h>
#include <windows.h>
#include "wireguard.h"
#include <string.h>
#include "tech_nevets_tvpn_wg_WireGuardJNI.h"

const char *APP_DIR =  "C:\\Program Files\\TVPN\\";

JNIEXPORT jint JNICALL Java_tech_nevets_tvpn_wg_WireGuardJNI_createTunnel(JNIEnv *env, jobject obj, jstring tunnelName) {
    const char *tunnelCharArr = (*env)->GetStringUTFChars(env, tunnelName, NULL);
    if (tunnelCharArr == NULL) {
        return 1;
    }

    char *initialServiceName = "WireGuardTunnel$";
    char *tnlName = malloc(strlen(initialServiceName) + strlen(tunnelCharArr) + 1);
    strcpy(tnlName, initialServiceName);
    strcat(tnlName, tunnelCharArr);

    char *initialDisplayName = "WireGuard Tunnel: ";
    char *displayName = malloc(strlen(initialDisplayName) + strlen(tunnelCharArr) + 1);
    strcpy(displayName, initialDisplayName);
    strcat(displayName, tunnelCharArr);

    SC_HANDLE hSCManager = OpenSCManager(
            NULL,
            NULL,
            SC_MANAGER_ALL_ACCESS
    );
    if (hSCManager == NULL) {
        printf("OpenSCManager failed (%lu)\n", GetLastError());
        return 2;
    }

    char *params = "TVPN.exe /installtunnelservice ";
    char *conf = ".conf";
    char *exePath = malloc(strlen(APP_DIR) + strlen(params) + strlen(tunnelCharArr) + strlen(conf) + 1);
    strcpy(exePath, APP_DIR);
    strcat(exePath, params);
    strcat(exePath, tunnelCharArr);
    strcat(exePath, conf);

    SC_HANDLE hService = CreateServiceA(
            hSCManager,
            tnlName,
            displayName,
            SERVICE_ALL_ACCESS,
            SERVICE_WIN32_OWN_PROCESS,
            SERVICE_AUTO_START,
            SERVICE_ERROR_NORMAL,
            exePath,
            NULL,
            NULL,
            "Nsi\0TcpIp\0\0",
            NULL,
            NULL
            );

    DWORD sid_info = SERVICE_SID_TYPE_UNRESTRICTED;

    if (!ChangeServiceConfig2A(hService, SERVICE_CONFIG_SERVICE_SID_INFO, &sid_info)) {
        printf("ChangeServiceConfig2W failed (%lu)\n", GetLastError());
        (*env)->ReleaseStringUTFChars(env, tunnelName, tunnelCharArr);
        free(tnlName);
        free(displayName);
        free(exePath);
        CloseServiceHandle(hSCManager);
        CloseServiceHandle(hService);
        return 3;
    }


    (*env)->ReleaseStringUTFChars(env, tunnelName, tunnelCharArr);
    free(tnlName);
    free(displayName);
    free(exePath);
    CloseServiceHandle(hSCManager);
    CloseServiceHandle(hService);

    return 0;
}

JNIEXPORT jint JNICALL Java_tech_nevets_tvpn_wg_WireGuardJNI_deleteTunnel(JNIEnv *env, jobject obj, jstring tunnelName) {
    const char *nativeString = (*env)->GetStringUTFChars(env, tunnelName, NULL);

    char *initialServiceName = "WireGuardTunnel$";
    char *tnlName = malloc(strlen(initialServiceName) + strlen(nativeString) + 1);
    strcpy(tnlName, initialServiceName);
    strcat(tnlName, nativeString);

    SC_HANDLE hSCManager = OpenSCManager(
            NULL,
            NULL,
            SC_MANAGER_ALL_ACCESS
            );
    if (hSCManager == NULL) {
        // Handle error
        return 1;
    }

    SC_HANDLE hService = OpenServiceA(hSCManager, tnlName, DELETE);
    if (hService != NULL) {
        if (!DeleteService(hService)) {
            // Handle error
            return 2;
        }
        CloseServiceHandle(hService);
    } else {
        // Handle error
        return 3;
    }

    CloseServiceHandle(hSCManager);
    (*env)->ReleaseStringUTFChars(env, tunnelName, nativeString);
    free(tnlName);

    return 0;
}

JNIEXPORT jint JNICALL Java_tech_nevets_tvpn_wg_WireGuardJNI_initializeWireGuard(JNIEnv *env, jobject obj, jstring configFilePath) {
    const char *nativeString = (*env)->GetStringUTFChars(env, configFilePath, 0);

    int confLength = strlen(nativeString);

    (*env)->ReleaseStringUTFChars(env, configFilePath, nativeString);

    return confLength;

//    const char *nativeConfigFilePath = (*env)->GetStringUTFChars(env, configFilePath, 0);

    // Load the WireGuard DLL dynamically
//    hInstLibrary = LoadLibrary("tunnel.dll");  // Adjust with the actual DLL name

//    if (hInstLibrary) {
        // Example: Load the WireGuard function from DLL
//        wg_func_t wg_init_func = (wg_func_t) GetProcAddress(hInstLibrary, "WireGuardInitFunction"); // Adjust based on the WireGuard API


//      if (wg_init_func) {
//            wg_init_func(nativeConfigFilePath);
//            (*env)->ReleaseStringUTFChars(env, configFilePath, nativeConfigFilePath);
//            return 0;  // Success
//      } else {
//            printf("Failed to get function from WireGuard DLL\n");
//            return -2;  // Failed to get function
//        }
//    } else {
//        printf("Failed to load WireGuard DLL\n");
//        return -1;  // Failed to load DLL
//    }
}

JNIEXPORT void JNICALL Java_tech_nevets_tvpn_wg_WireGuardJNI_cleanup(JNIEnv *env, jobject obj) {
    printf("Silly goose, this isn't implemented yet!\n");

//    if (hInstLibrary) {
//        FreeLibrary(hInstLibrary);
//    }
}
